#    Copyright 2013 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

module Astute
  class SshHardReboot

    def self.command
     <<-REBOOT_COMMAND
        if [ -r /etc/nailgun_systemtype ]; then
          NODE_TYPE=$(cat /etc/nailgun_systemtype)
        else
          NODE_TYPE="provisioning"
        fi

        # Check what was mounted to '/': drive (provisioned node)
        # or init ramdisk (bootsrapped/provisioning node)
        if grep -Eq 'root=[^[:blank:]]+' /proc/cmdline; then
          echo "Do not reboot $NODE_TYPE node using shell"
        else
          echo "Run node rebooting command using 'SB' to sysrq-trigger"
          echo "1" > /proc/sys/kernel/panic_on_oops
          echo "10" > /proc/sys/kernel/panic
          echo "b" > /proc/sysrq-trigger
        fi
      REBOOT_COMMAND
    end
  end
end
