#    Copyright 2014 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

module Astute
  class EnablePuppetDeploy < PreDeploymentAction

    # Unlock puppet (can be lock if puppet was killed by user)
    def process(deployment_info, context)
      nodes_uids = only_uniq_nodes(deployment_info).map{ |n| n['uid'] }
      puppetd = MClient.new(context, "puppetd", nodes_uids)
      puppetd.enable
    end #process
  end #class
end
